@@ -207,6 +207,19 @@ configuration_id = {
     ]
 }
 
+module_list = {
+    "type": "array",
+    "minItems": 0,
+    "items": {
+        "type": "object",
+        "required": ["id"],
+        "additionalProperties": True,
+        "properties": {
+            "id": uuid,
+        }
+    }
+}
+
 cluster = {
     "create": {
         "type": "object",
@@ -238,7 +251,8 @@ cluster = {
                                 "flavorRef": flavorref,
                                 "volume": volume,
                                 "nics": nics,
-                                "availability_zone": non_empty_string
+                                "availability_zone": non_empty_string,
+                                "modules": module_list,
                             }
                         }
                     }
@@ -334,7 +348,8 @@ instance = {
                             "version": non_empty_string
                         }
                     },
-                    "nics": nics
+                    "nics": nics,
+                    "modules": module_list
                 }
             }
         }
@@ -514,6 +529,108 @@ backup = {
     }
 }
 
+guest_log = {
+    "action": {
+        "name": "guest_log:action",
+        "type": "object",
+        "required": ["name"],
+        "properties": {
+            "name": non_empty_string,
+            "enable": boolean_string,
+            "disable": boolean_string,
+            "publish": boolean_string,
+            "discard": boolean_string
+        }
+    }
+}
+
+module_contents = {
+    "type": "string",
+    "minLength": 1,
+    "maxLength": 16777215,
+    "pattern": "^.*.+.*$"
+}
+
+module = {
+    "create": {
+        "name": "module:create",
+        "type": "object",
+        "required": ["module"],
+        "properties": {
+            "module": {
+                "type": "object",
+                "required": ["name", "module_type", "contents"],
+                "additionalProperties": True,
+                "properties": {
+                    "name": non_empty_string,
+                    "module_type": non_empty_string,
+                    "contents": module_contents,
+                    "description": non_empty_string,
+                    "datastore": {
+                        "type": "object",
+                        "properties": {
+                            "type": non_empty_string,
+                            "version": non_empty_string
+                        }
+                    },
+                    "auto_apply": boolean_string,
+                    "all_tenants": boolean_string,
+                    "visible": boolean_string,
+                    "live_update": boolean_string,
+                }
+            }
+        }
+    },
+    "update": {
+        "name": "module:update",
+        "type": "object",
+        "required": ["module"],
+        "properties": {
+            "module": {
+                "type": "object",
+                "required": [],
+                "additionalProperties": True,
+                "properties": {
+                    "name": non_empty_string,
+                    "type": non_empty_string,
+                    "contents": module_contents,
+                    "description": non_empty_string,
+                    "datastore": {
+                        "type": "object",
+                        "additionalProperties": True,
+                        "properties": {
+                            "type": non_empty_string,
+                            "version": non_empty_string
+                        }
+                    },
+                    "auto_apply": boolean_string,
+                    "all_tenants": boolean_string,
+                    "visible": boolean_string,
+                    "live_update": boolean_string,
+                }
+            }
+        }
+    },
+    "apply": {
+        "name": "module:apply",
+        "type": "object",
+        "required": ["modules"],
+        "properties": {
+            "modules": module_list,
+        }
+    },
+    "list": {
+        "name": "module:list",
+        "type": "object",
+        "required": [],
+        "properties": {
+            "module": uuid,
+            "from_guest": boolean_string,
+            "include_contents": boolean_string
+        }
+    },
+}
+
 configuration = {
     "create": {
         "name": "configuration:create",
