@@ -27,7 +27,6 @@ from trove.guestagent.db import models
 
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
-IGNORE_USERS_LIST = CONF.db2.ignore_users
 
 
 class DB2App(object):
@@ -44,8 +43,18 @@ class DB2App(object):
         LOG.debug("state_change_wait_time = %s." % self.state_change_wait_time)
         self.status = status
 
-    def complete_install_or_restart(self):
-        self.status.end_install_or_restart()
+    def update_hostname(self):
+        """
+        When DB2 server is installed, it uses the hostname of the
+        instance were the image was built. This needs to be updated
+        to reflect the guest instance.
+        """
+        LOG.debug("Update the hostname of the DB2 instance.")
+        try:
+            run_command(system.UPDATE_HOSTNAME,
+                        superuser='root')
+        except exception.ProcessExecutionError:
+            raise RuntimeError(_("Command to update the hostname failed."))
 
     def change_ownership(self, mount_point):
         """
@@ -104,7 +113,7 @@ class DB2App(object):
                 rd_instance.ServiceStatuses.RUNNING,
                 self.state_change_wait_time, update_db):
             LOG.error(_("Start of DB2 server instance failed."))
-            self.status.end_install_or_restart()
+            self.status.end_restart()
             raise RuntimeError(_("Could not start DB2."))
 
     def stop_db(self, update_db=False, do_not_start_on_reboot=False):
@@ -120,7 +129,7 @@ class DB2App(object):
                 rd_instance.ServiceStatuses.SHUTDOWN,
                 self.state_change_wait_time, update_db)):
             LOG.error(_("Could not stop DB2."))
-            self.status.end_install_or_restart()
+            self.status.end_restart()
             raise RuntimeError(_("Could not stop DB2."))
 
     def restart(self):
@@ -130,7 +139,7 @@ class DB2App(object):
             self.stop_db()
             self.start_db()
         finally:
-            self.status.end_install_or_restart()
+            self.status.end_restart()
 
 
 class DB2AppStatus(service.BaseDbStatus):
@@ -343,8 +352,9 @@ class DB2Admin(object):
                 LOG.debug("item = %r" % item)
                 user = item.split() if item != "" else None
                 LOG.debug("user = %r" % (user))
-                if user is not None and user[0] not in IGNORE_USERS_LIST \
-                        and user[1] == 'Y':
+                if (user is not None
+                    and (user[0] not in cfg.get_ignored_users(manager='db2')
+                         and user[1] == 'Y')):
                     userlist.append(user[0])
             result = iter(userlist)
 
