@@ -16,7 +16,7 @@
 
 from oslo_log import log as logging
 
-from trove.guestagent.strategy import Strategy
+from trove.common.strategies.strategy import Strategy
 
 LOG = logging.getLogger(__name__)
 
