@@ -14,17 +14,17 @@
 #    under the License.
 
 import os
-import proboscis
 import time as timer
 
+from oslo_config.cfg import NoSuchOptError
 from proboscis import asserts
+import swiftclient
 from troveclient.compat import exceptions
 
-from oslo_config.cfg import NoSuchOptError
 from trove.common import cfg
+from trove.common import exception
 from trove.common import utils
 from trove.common.utils import poll_until, build_polling_task
-from trove.common import exception
 from trove.tests.api.instances import instance_info
 from trove.tests.config import CONFIG
 from trove.tests.util import create_dbaas_client
@@ -33,19 +33,6 @@ from trove.tests.util.users import Requirements
 CONF = cfg.CONF
 
 
-class SkipKnownBug(proboscis.SkipTest):
-    """Skip test failures due to known bug(s).
-    These should get fixed sometime in the future.
-    """
-
-    def __init__(self, *bugs):
-        """
-        :param bugs:    One or more bug references (e.g. link, bug #).
-        """
-        bug_ref = '; '.join(map(str, bugs))
-        super(SkipKnownBug, self).__init__("Known bug: %s" % bug_ref)
-
-
 class TestRunner(object):
 
     """
@@ -82,9 +69,19 @@ class TestRunner(object):
     def __init__(self, sleep_time=10, timeout=1200):
         self.def_sleep_time = sleep_time
         self.def_timeout = timeout
+
         self.instance_info = instance_info
+        instance_info.dbaas_datastore = CONFIG.dbaas_datastore
+        instance_info.dbaas_datastore_version = CONFIG.dbaas_datastore_version
+        if self.VOLUME_SUPPORT:
+            instance_info.volume = {'size': CONFIG.get('trove_volume_size', 1)}
+        else:
+            instance_info.volume = None
+
         self.auth_client = create_dbaas_client(self.instance_info.user)
-        self.unauth_client = None
+        self._unauth_client = None
+        self._admin_client = None
+        self._swift_client = None
         self._test_helper = None
 
     @classmethod
@@ -154,12 +151,13 @@ class TestRunner(object):
     def test_helper(self, test_helper):
         self._test_helper = test_helper
 
-    def get_unauth_client(self):
-        if not self.unauth_client:
-            self.unauth_client = self._create_unauthorized_client()
-        return self.unauth_client
+    @property
+    def unauth_client(self):
+        if not self._unauth_client:
+            self._unauth_client = self._create_unauthorized_client()
+        return self._unauth_client
 
-    def _create_unauthorized_client(self, force=False):
+    def _create_unauthorized_client(self):
         """Create a client from a different 'unauthorized' user
         to facilitate negative testing.
         """
@@ -168,6 +166,44 @@ class TestRunner(object):
             requirements, black_list=[self.instance_info.user.auth_user])
         return create_dbaas_client(other_user)
 
+    @property
+    def admin_client(self):
+        if not self._admin_client:
+            self._admin_client = self._create_admin_client()
+        return self._admin_client
+
+    def _create_admin_client(self):
+        """Create a client from an admin user."""
+        requirements = Requirements(is_admin=True, services=["swift"])
+        admin_user = CONFIG.users.find_user(requirements)
+        return create_dbaas_client(admin_user)
+
+    @property
+    def swift_client(self):
+        if not self._swift_client:
+            self._swift_client = self._create_swift_client()
+        return self._swift_client
+
+    def _create_swift_client(self):
+        """Create a swift client from the admin user details."""
+        requirements = Requirements(is_admin=True, services=["swift"])
+        user = CONFIG.users.find_user(requirements)
+        os_options = {'region_name': CONFIG.trove_client_region_name}
+        return swiftclient.client.Connection(
+            authurl=CONFIG.nova_client['auth_url'],
+            user=user.auth_user,
+            key=user.auth_key,
+            tenant_name=user.tenant,
+            auth_version='2.0',
+            os_options=os_options)
+
+    def get_client_tenant(self, client):
+        tenant_name = client.real_client.client.tenant
+        service_url = client.real_client.client.service_url
+        su_parts = service_url.split('/')
+        tenant_id = su_parts[-1]
+        return tenant_name, tenant_id
+
     def assert_raises(self, expected_exception, expected_http_code,
                       client_cmd, *cmd_args, **cmd_kwargs):
         asserts.assert_raises(expected_exception, client_cmd,
@@ -187,7 +223,12 @@ class TestRunner(object):
 
     @property
     def is_using_existing_instance(self):
-        return os.environ.get(self.USE_INSTANCE_ID_FLAG, None) is not None
+        return self.has_env_flag(self.USE_INSTANCE_ID_FLAG)
+
+    @staticmethod
+    def has_env_flag(flag_name):
+        """Return whether a given flag was set."""
+        return os.environ.get(flag_name, None) is not None
 
     def get_existing_instance(self):
         if self.is_using_existing_instance:
@@ -198,8 +239,7 @@ class TestRunner(object):
 
     @property
     def has_do_not_delete_instance(self):
-        return os.environ.get(
-            self.DO_NOT_DELETE_INSTANCE_FLAG, None) is not None
+        return self.has_env_flag(self.DO_NOT_DELETE_INSTANCE_FLAG)
 
     def assert_instance_action(
             self, instance_ids, expected_states, expected_http_code):
@@ -232,7 +272,7 @@ class TestRunner(object):
                 self.fail(str(task.poll_exception()))
 
     def _assert_instance_states(self, instance_id, expected_states,
-                                fast_fail_status='ERROR',
+                                fast_fail_status=['ERROR', 'FAILED'],
                                 require_all_states=False):
         """Keep polling for the expected instance states until the instance
         acquires either the last or fast-fail state.
@@ -324,10 +364,11 @@ class TestRunner(object):
                    sleep_time=sleep_time, time_out=time_out)
 
     def _has_status(self, instance_id, status, fast_fail_status=None):
+        fast_fail_status = fast_fail_status or []
         instance = self.get_instance(instance_id)
         self.report.log("Polling instance '%s' for state '%s', was '%s'."
                         % (instance_id, status, instance.status))
-        if fast_fail_status and instance.status == fast_fail_status:
+        if instance.status in fast_fail_status:
             raise RuntimeError("Instance '%s' acquired a fast-fail status: %s"
                                % (instance_id, instance.status))
         return instance.status == status
@@ -365,7 +406,7 @@ class TestRunner(object):
         These are for internal use by the test framework and should
         not be changed by individual test-cases.
         """
-        database_def, user_def = self.build_helper_defs()
+        database_def, user_def, root_def = self.build_helper_defs()
         if database_def:
             self.report.log(
                 "Creating a helper database '%s' on instance: %s"
@@ -378,22 +419,33 @@ class TestRunner(object):
                 % (user_def['name'], user_def['password'], instance_id))
             self.auth_client.users.create(instance_id, [user_def])
 
+        if root_def:
+            # Not enabling root on a single instance of the cluster here
+            # because we want to test the cluster root enable instead.
+            pass
+
     def build_helper_defs(self):
         """Build helper database and user JSON definitions if credentials
         are defined by the helper.
         """
         database_def = None
-        user_def = None
+
+        def _get_credentials(creds):
+            if creds:
+                username = creds.get('name')
+                if username:
+                    password = creds.get('password', '')
+                    return {'name': username, 'password': password,
+                            'databases': [{'name': database}]}
+            return None
+
         credentials = self.test_helper.get_helper_credentials()
         if credentials:
             database = credentials.get('database')
             if database:
                 database_def = {'name': database}
+        credentials_root = self.test_helper.get_helper_credentials_root()
 
-            username = credentials.get('name')
-            if username:
-                password = credentials.get('password', '')
-                user_def = {'name': username, 'password': password,
-                            'databases': [{'name': database}]}
-
-        return database_def, user_def
+        return (database_def,
+                _get_credentials(credentials),
+                _get_credentials(credentials_root))
