@@ -28,11 +28,11 @@ class TestClusterModel(trove_testtools.TestCase):
     @patch.object(datastore_models.DatastoreVersion, 'load_by_uuid')
     @patch.object(models.DBCluster, 'find_by')
     def test_load(self, mock_find_by, mock_load_dsv_by_uuid, mock_ds_load):
-        context = Mock()
+        context = trove_testtools.TroveTestContext(self)
         id = Mock()
 
         dsv = Mock()
         dsv.manager = 'mongodb'
         mock_load_dsv_by_uuid.return_value = dsv
         cluster = models.Cluster.load(context, id)
-        self.assertTrue(isinstance(cluster, MongoDbCluster))
+        self.assertIsInstance(cluster, MongoDbCluster)
