@@ -13,7 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import itertools
 import os
 import re
 import stat
@@ -21,11 +20,13 @@ import tempfile
 
 from mock import call, patch
 from oslo_concurrency.processutils import UnknownArgumentError
+import six
 from testtools import ExpectedException
 
 from trove.common import exception
 from trove.common.stream_codecs import (
-    IdentityCodec, IniCodec, JsonCodec, PropertiesCodec, YamlCodec)
+    Base64Codec, IdentityCodec, IniCodec, JsonCodec,
+    KeyValueCodec, PropertiesCodec, YamlCodec)
 from trove.common import utils
 from trove.guestagent.common import guestagent_utils
 from trove.guestagent.common import operating_system
@@ -35,6 +36,16 @@ from trove.tests.unittests import trove_testtools
 
 class TestOperatingSystem(trove_testtools.TestCase):
 
+    def test_base64_codec(self):
+        data = "Line 1\nLine 2\n"
+        self._test_file_codec(data, Base64Codec())
+
+        data = "TGluZSAxCkxpbmUgMgo="
+        self._test_file_codec(data, Base64Codec(), reverse_encoding=True)
+
+        data = "5Am9+y0wTwqUx39sMMBg3611FWg="
+        self._test_file_codec(data, Base64Codec(), reverse_encoding=True)
+
     def test_identity_file_codec(self):
         data = ("Lorem Ipsum, Lorem Ipsum\n"
                 "Lorem Ipsum, Lorem Ipsum\n"
@@ -97,13 +108,21 @@ class TestOperatingSystem(trove_testtools.TestCase):
                 'key7': 0,
                 'key8': None,
                 'key9': [['str1', 'str2'], ['str3', 'str4']],
-                'key10': [['str1', 'str2', 'str3'], ['str3', 'str4'], 'str5']
+                'key10': [['str1', 'str2', 'str3'], ['str3', 'str4'], 'str5'],
+                'key11': True
                 }
 
         self._test_file_codec(data, PropertiesCodec())
         self._test_file_codec(data, PropertiesCodec(
             string_mappings={'yes': True, 'no': False, "''": None}))
 
+    def test_key_value_file_codec(self):
+        data = {'key1': 'value1',
+                'key2': 'value2',
+                'key3': 'value3'}
+
+        self._test_file_codec(data, KeyValueCodec())
+
     def test_json_file_codec(self):
         data = {"Section1": 's1v1',
                 "Section2": {"s2k1": '1',
@@ -116,21 +135,31 @@ class TestOperatingSystem(trove_testtools.TestCase):
 
     def _test_file_codec(self, data, read_codec, write_codec=None,
                          expected_data=None,
-                         expected_exception=None):
+                         expected_exception=None,
+                         reverse_encoding=False):
         write_codec = write_codec or read_codec
 
         with tempfile.NamedTemporaryFile() as test_file:
+            encode = True
+            decode = True
+            if reverse_encoding:
+                encode = False
+                decode = False
             if expected_exception:
                 with expected_exception:
                     operating_system.write_file(test_file.name, data,
-                                                codec=write_codec)
+                                                codec=write_codec,
+                                                encode=encode)
                     operating_system.read_file(test_file.name,
-                                               codec=read_codec)
+                                               codec=read_codec,
+                                               decode=decode)
             else:
                 operating_system.write_file(test_file.name, data,
-                                            codec=write_codec)
+                                            codec=write_codec,
+                                            encode=encode)
                 read = operating_system.read_file(test_file.name,
-                                                  codec=read_codec)
+                                                  codec=read_codec,
+                                                  decode=decode)
                 if expected_data is not None:
                     self.assertEqual(expected_data, read)
                 else:
@@ -627,6 +656,74 @@ class TestOperatingSystem(trove_testtools.TestCase):
                               "Got unknown keyword args: {'_unknown_kw': 0}"),
             'path', 'usr', None, _unknown_kw=0)
 
+    def test_change_user_group(self):
+        self._assert_execute_call(
+            [['usermod', '-a', '-G', 'user', 'group']],
+            [{'run_as_root': True, 'root_helper': 'sudo'}],
+            operating_system.change_user_group, None, 'group', 'user',
+            as_root=True)
+
+        self._assert_execute_call(
+            [['usermod', '-a', '-G', 'user', 'group']],
+            [{'run_as_root': True, 'root_helper': 'sudo'}],
+            operating_system.change_user_group, None, 'group', 'user',
+            append=True, add_group=True, as_root=True)
+
+        self._assert_execute_call(
+            [['usermod', '-a', '-G', 'user', 'group']],
+            [{'timeout': 100}],
+            operating_system.change_user_group, None, 'group', 'user',
+            timeout=100)
+
+        self._assert_execute_call(
+            [['usermod', '-a', '-G', 'user', 'group']],
+            [{'run_as_root': True, 'root_helper': "sudo", 'timeout': None}],
+            operating_system.change_user_group, None, 'group', 'user',
+            timeout=None, as_root=True)
+
+        self._assert_execute_call(
+            None, None,
+            operating_system.change_user_group,
+            ExpectedException(exception.UnprocessableEntity,
+                              "Missing user."), '', 'group')
+
+        self._assert_execute_call(
+            None, None,
+            operating_system.change_user_group,
+            ExpectedException(exception.UnprocessableEntity,
+                              "Missing user."), None, 'group')
+
+        self._assert_execute_call(
+            None, None,
+            operating_system.change_user_group,
+            ExpectedException(exception.UnprocessableEntity,
+                              "Missing group."), 'user', '')
+
+        self._assert_execute_call(
+            None, None,
+            operating_system.change_user_group,
+            ExpectedException(exception.UnprocessableEntity,
+                              "Missing group."), 'user', None)
+
+        self._assert_execute_call(
+            None, None,
+            operating_system.change_user_group,
+            ExpectedException(exception.UnprocessableEntity,
+                              "Missing user."), '', '')
+
+        self._assert_execute_call(
+            None, None,
+            operating_system.change_user_group,
+            ExpectedException(exception.UnprocessableEntity,
+                              "Missing user."), None, None)
+
+        self._assert_execute_call(
+            None, None,
+            operating_system.change_user_group,
+            ExpectedException(UnknownArgumentError,
+                              "Got unknown keyword args: {'_unknown_kw': 0}"),
+            'user', 'add_group', _unknown_kw=0)
+
     def test_create_directory(self):
         self._assert_execute_call(
             [['mkdir', '-p', 'path']],
@@ -713,8 +810,33 @@ class TestOperatingSystem(trove_testtools.TestCase):
                               "Got unknown keyword args: {'_unknown_kw': 0}"),
             'path', _unknown_kw=0)
 
+    def test_exists(self):
+        self.assertFalse(
+            operating_system.exists(tempfile.gettempdir(), is_directory=False))
+        self.assertTrue(
+            operating_system.exists(tempfile.gettempdir(), is_directory=True))
+
+        with tempfile.NamedTemporaryFile() as test_file:
+            self.assertTrue(
+                operating_system.exists(test_file.name, is_directory=False))
+            self.assertFalse(
+                operating_system.exists(test_file.name, is_directory=True))
+
+        self._assert_execute_call(
+            [['test -f path && echo 1 || echo 0']],
+            [{'shell': True, 'check_exit_code': False,
+              'run_as_root': True, 'root_helper': 'sudo'}],
+            operating_system.exists, None, 'path', is_directory=False,
+            as_root=True)
+        self._assert_execute_call(
+            [['test -d path && echo 1 || echo 0']],
+            [{'shell': True, 'check_exit_code': False,
+              'run_as_root': True, 'root_helper': 'sudo'}],
+            operating_system.exists, None, 'path', is_directory=True,
+            as_root=True)
+
     def _assert_execute_call(self, exec_args, exec_kwargs,
-                             fun, return_value, *args, **kwargs):
+                             func, return_value, *args, **kwargs):
         """
         Execute a function with given arguments.
         Assert a return value and appropriate sequence of calls to the
@@ -732,8 +854,8 @@ class TestOperatingSystem(trove_testtools.TestCase):
                                   'utils.execute_with_timeout'.
         :type exec_kwargs:        list-of-dicts
 
-        :param fun:               Tested function call.
-        :type fun:                callable
+        :param func:              Tested function call.
+        :type func:               callable
 
         :param return_value:      Expected return value or exception
                                   from the tested call if any.
@@ -746,17 +868,18 @@ class TestOperatingSystem(trove_testtools.TestCase):
         :type kwargs:             dict
         """
 
-        with patch.object(utils, 'execute_with_timeout') as exec_call:
+        with patch.object(utils, 'execute_with_timeout',
+                          return_value=('0', '')) as exec_call:
             if isinstance(return_value, ExpectedException):
                 with return_value:
-                    fun(*args, **kwargs)
+                    func(*args, **kwargs)
             else:
-                actual_value = fun(*args, **kwargs)
+                actual_value = func(*args, **kwargs)
                 if return_value is not None:
                     self.assertEqual(return_value, actual_value,
                                      "Return value mismatch.")
                 expected_calls = []
-                for arg, kw in itertools.izip(exec_args, exec_kwargs):
+                for arg, kw in six.moves.zip(exec_args, exec_kwargs):
                     expected_calls.append(call(*arg, **kw))
 
                 self.assertEqual(expected_calls, exec_call.mock_calls,
@@ -819,6 +942,10 @@ class TestOperatingSystem(trove_testtools.TestCase):
             config_file = operating_system.file_discovery(
                 ["/etc/mongodb.conf", "/etc/mongod.conf"])
         self.assertEqual('/etc/mongod.conf', config_file)
+        with patch.object(os.path, 'isfile', side_effect=[False]):
+            config_file = operating_system.file_discovery(
+                ["/etc/mongodb.conf"])
+        self.assertEqual('', config_file)
 
     def test_list_files_in_directory(self):
         root_path = tempfile.mkdtemp()
@@ -828,39 +955,67 @@ class TestOperatingSystem(trove_testtools.TestCase):
                 root_path, 3, 3, ['txt', 'py', ''], 1, all_paths)
 
             # All files in the top directory.
-            self._assert_list_files(root_path, False, None, all_paths, 9)
+            self._assert_list_files(
+                root_path, False, None, False, all_paths, 9)
+
+            # All files & directories in the top directory.
+            self._assert_list_files(
+                root_path, False, None, True, all_paths, 10)
 
             # All files recursive.
-            self._assert_list_files(root_path, True, None, all_paths, 27)
+            self._assert_list_files(
+                root_path, True, None, False, all_paths, 27)
+
+            # All files & directories recursive.
+            self._assert_list_files(
+                root_path, True, None, True, all_paths, 29)
 
             # Only '*.txt' in the top directory.
-            self._assert_list_files(root_path, False, '.*\.txt$', all_paths, 3)
+            self._assert_list_files(
+                root_path, False, '.*\.txt$', False, all_paths, 3)
+
+            # Only '*.txt' (including directories) in the top directory.
+            self._assert_list_files(
+                root_path, False, '.*\.txt$', True, all_paths, 3)
 
             # Only '*.txt' recursive.
-            self._assert_list_files(root_path, True, '.*\.txt$', all_paths, 9)
+            self._assert_list_files(
+                root_path, True, '.*\.txt$', True, all_paths, 9)
+
+            # Only '*.txt' (including directories) recursive.
+            self._assert_list_files(
+                root_path, True, '.*\.txt$', False, all_paths, 9)
 
             # Only extension-less files in the top directory.
-            self._assert_list_files(root_path, False, '[^\.]*$', all_paths, 3)
+            self._assert_list_files(
+                root_path, False, '[^\.]*$', False, all_paths, 3)
 
             # Only extension-less files recursive.
-            self._assert_list_files(root_path, True, '[^\.]*$', all_paths, 9)
+            self._assert_list_files(
+                root_path, True, '[^\.]*$', False, all_paths, 9)
 
             # Non-existing extension in the top directory.
-            self._assert_list_files(root_path, False, '.*\.bak$', all_paths, 0)
+            self._assert_list_files(
+                root_path, False, '.*\.bak$', False, all_paths, 0)
 
             # Non-existing extension recursive.
-            self._assert_list_files(root_path, True, '.*\.bak$', all_paths, 0)
+            self._assert_list_files(
+                root_path, True, '.*\.bak$', False, all_paths, 0)
         finally:
             try:
                 os.remove(root_path)
             except Exception:
                 pass  # Do not fail in the cleanup.
 
-    def _assert_list_files(self, root, recursive, pattern, all_paths, count):
+    def _assert_list_files(self, root, recursive, pattern, include_dirs,
+                           all_paths, count):
         found = operating_system.list_files_in_directory(
-            root, recursive=recursive, pattern=pattern)
+            root, recursive=recursive, pattern=pattern,
+            include_dirs=include_dirs)
         expected = {
-            path for path in all_paths if (
+            path for path in filter(
+                lambda item: include_dirs or not os.path.isdir(item),
+                all_paths) if (
                 (recursive or os.path.dirname(path) == root) and (
                     not pattern or re.match(
                         pattern, os.path.basename(path))))}
@@ -880,6 +1035,7 @@ class TestOperatingSystem(trove_testtools.TestCase):
 
         if level < num_levels:
             path = tempfile.mkdtemp(dir=root_path)
+            created_paths.add(path)
             self._create_temp_fs_structure(
                 path, num_levels, num_files_per_extension,
                 file_extensions, level + 1, created_paths)
