@@ -16,14 +16,13 @@
 import uuid
 
 from mock import MagicMock, patch, ANY
-from novaclient.v2 import Client
+from novaclient.client import Client
 from novaclient.v2.flavors import FlavorManager, Flavor
 from novaclient.v2.servers import Server, ServerManager
 from oslo_config import cfg
 from testtools.matchers import Equals, Is, Not
 
 from trove.backup.models import Backup
-from trove.common.context import TroveContext
 from trove.common import exception
 from trove.common import instance as rd_instance
 from trove.common import remote
@@ -69,7 +68,7 @@ class MockMgmtInstanceTest(trove_testtools.TestCase):
         super(MockMgmtInstanceTest, cls).tearDownClass()
 
     def setUp(self):
-        self.context = TroveContext()
+        self.context = trove_testtools.TroveTestContext(self)
         self.context.auth_token = 'some_secret_password'
         self.client = MagicMock(spec=Client)
         self.server_mgr = MagicMock(spec=ServerManager)
@@ -80,9 +79,11 @@ class MockMgmtInstanceTest(trove_testtools.TestCase):
             remote, 'create_admin_nova_client', return_value=self.client)
         self.addCleanup(self.admin_client_patch.stop)
         self.admin_client_patch.start()
-        CONF.set_override('host', 'test_host')
-        CONF.set_override('exists_notification_interval', 1)
-        CONF.set_override('notification_service_id', {'mysql': '123'})
+        CONF.set_override('host', '127.0.0.1', enforce_type=True)
+        CONF.set_override('exists_notification_interval', 1,
+                          enforce_type=True)
+        CONF.set_override('notification_service_id', {'mysql': '123'},
+                          enforce_type=True)
 
         super(MockMgmtInstanceTest, self).setUp()
 
@@ -121,7 +122,8 @@ class TestNotificationTransformer(MockMgmtInstanceTest):
     def setUpClass(cls):
         super(TestNotificationTransformer, cls).setUpClass()
 
-    def test_tranformer(self):
+    @patch('trove.instance.models.LOG')
+    def test_transformer(self, mock_logging):
         status = rd_instance.ServiceStatuses.BUILDING.api_status
         instance, service_status = self.build_db_instance(
             status, InstanceTasks.BUILDING)
@@ -144,7 +146,8 @@ class TestNotificationTransformer(MockMgmtInstanceTest):
         self.assertThat(transformer._get_service_id('mysql', id_map),
                         Equals('123'))
 
-    def test_get_service_id_unknown(self):
+    @patch('trove.extensions.mgmt.instances.models.LOG')
+    def test_get_service_id_unknown(self, mock_logging):
         id_map = {
             'mysql': '123',
             'percona': 'abc'
@@ -182,7 +185,7 @@ class TestNovaNotificationTransformer(MockMgmtInstanceTest):
             self.assertThat(transformer._lookup_flavor('2'),
                             Equals('unknown'))
 
-    def test_tranformer(self):
+    def test_transformer(self):
         status = rd_instance.ServiceStatuses.BUILDING.api_status
         instance, service_status = self.build_db_instance(
             status, InstanceTasks.BUILDING)
@@ -220,7 +223,8 @@ class TestNovaNotificationTransformer(MockMgmtInstanceTest):
                 self.assertThat(payload['service_id'], Equals('123'))
         self.addCleanup(self.do_cleanup, instance, service_status)
 
-    def test_tranformer_invalid_datastore_manager(self):
+    @patch('trove.extensions.mgmt.instances.models.LOG')
+    def test_transformer_invalid_datastore_manager(self, mock_logging):
         status = rd_instance.ServiceStatuses.BUILDING.api_status
         instance, service_status = self.build_db_instance(
             status, InstanceTasks.BUILDING)
@@ -265,7 +269,7 @@ class TestNovaNotificationTransformer(MockMgmtInstanceTest):
         version.update(manager='mysql')
         self.addCleanup(self.do_cleanup, instance, service_status)
 
-    def test_tranformer_shutdown_instance(self):
+    def test_transformer_shutdown_instance(self):
         status = rd_instance.ServiceStatuses.SHUTDOWN.api_status
         instance, service_status = self.build_db_instance(status)
         service_status.set_status(rd_instance.ServiceStatuses.SHUTDOWN)
@@ -293,7 +297,7 @@ class TestNovaNotificationTransformer(MockMgmtInstanceTest):
                                       for db in payloads])
         self.addCleanup(self.do_cleanup, instance, service_status)
 
-    def test_tranformer_no_nova_instance(self):
+    def test_transformer_no_nova_instance(self):
         status = rd_instance.ServiceStatuses.SHUTDOWN.api_status
         instance, service_status = self.build_db_instance(status)
         service_status.set_status(rd_instance.ServiceStatuses.SHUTDOWN)
@@ -318,7 +322,7 @@ class TestNovaNotificationTransformer(MockMgmtInstanceTest):
                                       for db in payloads])
         self.addCleanup(self.do_cleanup, instance, service_status)
 
-    def test_tranformer_flavor_cache(self):
+    def test_transformer_flavor_cache(self):
         status = rd_instance.ServiceStatuses.BUILDING.api_status
         instance, service_status = self.build_db_instance(
             status, InstanceTasks.BUILDING)
